@@ -1,9 +1,10 @@
 package YellowBot::API::Request;
-BEGIN {
-  $YellowBot::API::Request::VERSION = '0.94';
+{
+  $YellowBot::API::Request::VERSION = '0.96';
 }
 use Moose;
 use URI ();
+use Carp qw(croak);
 use Digest::SHA qw(hmac_sha256_hex);
 use HTTP::Request::Common qw(POST);
 use namespace::clean -except => 'meta';
@@ -32,29 +33,51 @@ sub _query {
 
     my $api_secret = delete $args{api_secret};
 
-    $args{api_ts}  = time;
-    $args{api_sig} = hmac_sha256_hex(_get_parameter_string(\%args), $api_secret);
+    $args{api_ts}  ||= time;
+    $args{api_sig}   = hmac_sha256_hex(_get_parameter_string(\%args), $api_secret);
 
     return %args;
 }
 
-sub http_request {
+sub _signed_args {
+    return shift->args;
+}
+
+sub _signed_query_form {
     my $self = shift;
-    my $uri = URI->new( $self->api->server );
-    $uri->path("/api/" . $self->method );
 
-    my %args = %{ $self->args };
-
-    my %content = _query
+    my $signed_args = $self->_signed_args;
+    return _query
       (
-       %args,
+       %$signed_args,
        api_key    => $self->api->api_key,
        api_secret => $self->api->api_secret,
       );
 
-    my $request = POST( $uri, [ %content ],
-                        'Content-Type' => 'form-data',
-                      );
+}
+
+sub _build_uri {
+    my $self = shift;
+    my $uri = URI->new( $self->api->server );
+    $uri->path("/api/" . $self->method);
+    return $uri;
+}
+
+sub _build_request {
+    my $self = shift;
+    
+    my $uri = $self->_build_uri;
+    my %content = $self->_signed_query_form;
+
+    return POST($uri, [%content], 'Content-Type' => 'form-data');
+}
+
+sub http_request {
+    my $self = shift;
+
+    my $request = $self->_build_request;
+
+    #$request->dump( prefix => ' > ' );
 
     if ($ENV{API_DEBUG}) {
         require Data::Dumper;
@@ -64,13 +87,32 @@ sub http_request {
     return $request;
 }
 
+sub _file_as_value {
+    my $v  = shift;
+    my $fn = $v->[1];
+    unless (defined $fn) {    # Infer from local file name
+        $fn = $v->[0];
+        $fn =~ s,.*/,, if defined $fn;
+    }
+    unless ($fn) {
+        croak "File uploads must have file names";
+    }
+    return $fn;
+}
+# Based on HTTP::Request::form_data() handling of uploads
+# according to the specification [ $file, $usename, @headers ]
+
 sub _get_parameter_string {
     my $args = shift;
 
     my $str = "";
     for my $key (sort {$a cmp $b} keys %{$args}) {
         next if $key eq 'api_sig';
-        my $value = (defined($args->{$key})) ? $args->{$key} : "";
+        my $v = $args->{$key};
+        my $value =
+          defined($v)
+          ? (ref($v) ? _file_as_value($v) : $v)
+          : "";
         $str .= $key . $value;
     }
     return $str;
